@@ -1,6 +1,5 @@
 package MozRepl::RemoteObject;
 use strict;
-
 use JSON;
 use Carp qw(croak cluck);
 use MozRepl;
@@ -39,7 +38,7 @@ MozRepl::RemoteObject - treat Javascript objects as Perl objects
 =cut
 
 use vars qw[$VERSION $objBridge];
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 # This should go into __setup__ and attach itself to $repl as .link()
 $objBridge = <<JS;
@@ -121,13 +120,17 @@ JS
 sub to_perl {
     my ($self,$js) = @_;
     local $_ = $js;
+    s/^(\.+\>)+//; # remove Mozrepl continuation prompts
     s/^"//;
     s/"$//;
     # reraise JS errors from perspective of caller
     if (/^!!!\s+(.*)$/m) {
         croak "MozRepl::RemoteObject: $1";
     };
-    $self->json->decode($_);
+    #warn "[[$_]]";
+    s/\\x/\\\\x/g; # effin' .toSource() sends us \xHH escapes, and JSON doesn't
+    # know what to do with them. So I pass them through unharmed :-(
+    $self->json->decode($_)
 };
 
 # Unwrap the result, will in the future also be used
@@ -192,6 +195,7 @@ call C<< ->install_bridge >> as follows:
 sub install_bridge {
     my ($package, %options) = @_;
     $options{ repl } ||= $ENV{MOZREPL};
+    $options{ log } ||= [qw/ error/];
     
     if (! ref $options{repl}) { # we have host:port
         my @host_port;
@@ -211,14 +215,13 @@ sub install_bridge {
                     binmode => 1,
                 }
             },
-            log => [qw/ error/],
-            #log => [qw/ debug error/],
+            log => $options{ log },
             plugins => { plugins => [qw[ JSON2 ]] }, # I'm loading my own JSON serializer
         });
     };
     
     my $rn = $options{repl}->repl;
-    $options{ json } ||= JSON->new->allow_nonref; # ->utf8;
+    $options{ json } ||= JSON->new->allow_nonref->ascii; #->utf8;
 
     # Load the JS side of the JS <-> Perl bridge
     for my $c ($objBridge) {
@@ -294,6 +297,7 @@ sub js_call_to_perl_struct {
     my ($self,$js) = @_;
     my $repl = $self->repl;
     $js = "JSON.stringify( function(){ var res = $js; return { result: res }}())";
+    #warn "<<$js>>";
     my $d = $self->to_perl($repl->execute($js));
     $d->{result}
 };
@@ -748,7 +752,7 @@ on HTMLdocument nodes or their children.
 
 sub __click {
     my ($self) = @_; # $self is a HTMLdocument or a descendant!
-    my $click = $self->declare(<<'JS');
+    my $click = $self->bridge->declare(<<'JS');
     function(target) {
         var event = content.document.createEvent('MouseEvents');
         event.initMouseEvent('click', true, true, window,
@@ -953,22 +957,25 @@ __END__
 
 The communication with the MozRepl plugin is done
 through 7bit safe ASCII. The received bytes are supposed
-to be UTF-8, but this seems not always to be the case.
+to be UTF-8, but this seems not always to be the case,
+so the JSON encoder on the Javascript side also
+uses a 7bit safe encoding.
 
 Currently there is no way to specify a different encoding
 on the fly. You have to replace or reconfigure
 the JSON object in the constructor.
 
-You can toggle the utf8'ness by calling
-
-  $bridge->json->utf8;
-
 =head1 TODO
 
 =over 4
 
 =item *
 
+Implement C<EXISTS()> so we can use
+C<< exists $foo->{onClick} >>.
+
+=item *
+
 For tests that connect to the outside world,
 check/ask whether we're allowed to. If running
 automated, skip.
@@ -1091,6 +1098,17 @@ with other mainloops.
 
 This would lead to implementing a full two-way message bus.
 
+=item *
+
+Potentially, C<repl.print()> on the Javascript side can trigger
+an event. This would mean that we need asynchronous IO
+between Perl and JS, and potentially L<AnyEvent>.
+
+=item *
+
+Consider implementing a mozrepl "interactor" to remove
+the prompting of C<mozrepl> alltogether.
+
 =back
 
 =head1 SEE ALSO
